/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sysexits.h>

#include "zm_mem.h"
#include "zm_data_buffer_impl.h"

/*
  Own allocator
 */
typedef struct _own_allocator
{
	uint32_t    allocs;
	uint32_t    reallocs;
	uint32_t    frees;
} OWN_ALLOCATOR;

/*
  Allocate memory
 */
static void * own_allocate(void    * opaque,
                           size_t    size)
{
	OWN_ALLOCATOR * own = (OWN_ALLOCATOR *)opaque;
	own -> allocs += 1;

	return malloc(size);
}

/*
 * Reallocate memory
 */
static void * own_reallocate(void    * opaque,
                             void    * ptr,
                             size_t    size)
{
	OWN_ALLOCATOR * own = (OWN_ALLOCATOR *)opaque;
	own -> reallocs += 1;

	return realloc(ptr, size);
}

/*
 * Deallocate memory
 */
static void own_deallocate(void  * opaque,
                           void  * ptr)

{
	OWN_ALLOCATOR * own = (OWN_ALLOCATOR *)opaque;
	own -> frees += 1;

	free(ptr);
}

int main(int argc, char ** argv)
{
	ZM_DATA_BUFFER     buf;

	/*
	 * Own memory allocation object
	 * We have to create functions for allocation, reallocation and deallocation memory
	 * We also want to know some stats about memory consumtion. So, create some stats object.
	 */
	OWN_ALLOCATOR      alloc_stat;
	ZM_ALLOCATOR       alloc;
	alloc_stat.allocs   = 0;
	alloc_stat.reallocs = 0;
	alloc_stat.frees    = 0;

	/* Our opaque data for functions         */
 	alloc.opaque     = &alloc_stat;
 	/* Allocate memory; see stdlib malloc    */
 	alloc.allocate   = own_allocate;
 	/* Reallocate memory; see stdlib realloc */
 	alloc.reallocate = own_reallocate;
 	/* Free memory; see stdlib free          */
 	alloc.deallocate = own_deallocate;

	/* Fill some data */
	zm_data_buffer_create_impl(&buf, &alloc);
	zm_data_buffer_append_str(&buf, "Data buffer test: ");
	zm_data_buffer_append_str(&buf, "Sed ut perspiciatis, unde omnis iste natus ");
	zm_data_buffer_append_str(&buf, "error sit voluptatem accusantium doloremque ");
	zm_data_buffer_append_str(&buf, "laudantium, totam rem aperiam eaque ipsa, quae ");
	zm_data_buffer_append_str(&buf, "ab illo inventore veritatis et quasi architecto ");
	zm_data_buffer_append_str(&buf, "beatae vitae dicta sunt, explicabo. Nemo enim ipsam voluptatem, ");
	zm_data_buffer_append_str(&buf, "quia voluptas sit, aspernatur aut odit aut fugit, ");
	zm_data_buffer_append_str(&buf, "sed quia consequuntur magni dolores eos, qui ratione ");
	zm_data_buffer_append_str(&buf, "voluptatem sequi nesciunt, neque porro quisquam est, ");
	zm_data_buffer_append_str(&buf, "qui dolorem ipsum, quia dolor sit, amet, consectetur, ");
	zm_data_buffer_append_str(&buf, "adipisci velit, sed quia non numquam eius modi tempora ");
	zm_data_buffer_append_str(&buf, "incidunt, ut labore et dolore magnam aliquam quaerat voluptatem. ");
	zm_data_buffer_destroy(&buf);

	/* Print usage statistics */
	fprintf(stdout, "Allocs: %u, Reallocs %u, Frees %u\n", alloc_stat.allocs, alloc_stat.reallocs, alloc_stat.frees);

return EX_OK;
}
/* End. */
