/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#ifndef _ZM_ADDRESS_PARSER_H__
#define _ZM_ADDRESS_PARSER_H__ 1

/**
 * @file zm_address_parser.h
 * @brief Address parser
 */
#include "zm_address_parser_defs.h"
#include "zm_data_buffer.h"
#include "zm_email_address_defs.h"
#include "zm_iconv_defs.h"
#include "zm_mem_defs.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @struct _zm_address_parser zm_address_parser.h <zm_address_parser.h>
 * @brief Address parser
 */
struct _zm_address_parser
{
	/** Text converter               */
	ZM_ICONV          * iconv;
	/** Comment                      */
	ZM_DATA_BUFFER      comment;
	/** Current address              */
	ZM_EMAIL_ADDRESS  * curr;
	/** Opaque data allocator        */
	ZM_ALLOCATOR      * alloc;
	/* Flags                         */
	uint32_t            flags;
};

/**
 * @brief Create email address parser
 */
ZM_ADDRESS_PARSER * zm_address_parser_create(ZM_ADDRESS_PARSER  * address_parser,
                                             ZM_ICONV           * iconv,
                                             ZM_DATA_BUFFER     * buffer);

/**
 * @brief Parse email address
 */
int32_t zm_address_parser_parse_address_list(ZM_ADDRESS_PARSER    * address_parser,
                                             const unsigned char ** src,
                                             uint64_t             * src_len,
                                             ZM_EMAIL_ADDRESS    ** email_address);

/**
 * @brief Destroy address parser
 */
void zm_address_parser_destroy(ZM_ADDRESS_PARSER  * address_parser);


#ifdef __cplusplus
}
#endif

#endif /* _ZM_ADDRESS_PARSER_H__ */
/* End. */
