/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#ifndef _ZM_ALIASES_H__
#define _ZM_ALIASES_H__ 1

/**
  @file zm_aliases.h
  @brief Zero MIME aliases
*/

#include "zm_mem_defs.h"
#include "zm_aliases_defs.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @brief Load custom aliases database
 * @param file - File to load
 * @param alloc - Memory allocator
 * @param aliases - List of aliasesations [out]
 * @return 0 - if success, -1 - otherwise
 */
int32_t zm_load_aliases(const char     * file,
                        ZM_ALLOCATOR   * alloc,
                        ZM_ALIASES    ** aliases);

/**
 * @brief Print aliases database
 * @param aliases - List of aliases
 */
void zm_print_aliases(ZM_ALIASES  * aliases);

/**
 * @brief Free aliases
 * @param aliases - List of aliases
 */
void zm_free_aliases(ZM_ALIASES  * aliases);

/**
 * @brief Parse aliases
 * @param aliases - List of aliases
 * @param alias - Alias
 * @param alias_len - Alias length
 * @param name - Name or NULL [out]
 * @return 0 - if success, -1 - otherwise
 */
int32_t zm_parse_alias(ZM_ALIASES      * aliases,
                       const char      * alias,
                       const uint32_t    alias_len,
                       const char     ** name);

#ifdef __cplusplus
}
#endif

#endif /* _ZM_ALIASES_H__ */
/* End. */
