/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#ifndef _ZM_BLOCK_H__
#define _ZM_BLOCK_H__ 1

/**
  @file zm_block.h
  @brief Zero MIME data block
*/

#include "zm_block_defs.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @struct _zm_block zm_block.h <zm_block.h>
 * @brief Block of data
 */
struct ZM_API _zm_block
{
	/** Start offset */
	uint64_t    start;
	/** End offset   */
	uint64_t    end;
};

/**
 * @brief Create data block
 * @param block - Data block
 * @param start - Start offset
 * @param end - End offset
 */
void zm_block_create(ZM_BLOCK  * block,
                     uint64_t    start,
                     uint64_t    end);

/**
 * @brief Create empty block
 * @param block - Data block
 */
void zm_block_zero(ZM_BLOCK  * block);

/**
 * @brief Copy block
 * @param block_dst - Destination block
 * @param block_src - Source block
 */
void zm_block_copy(ZM_BLOCK        * block_dst,
                   const ZM_BLOCK  * block_src);

/**
 * @brief Get data from block
 * @param data - Data block
 * @param len - Data length [out]
 * @return Pointer to data block
 */
const void * zm_block_get_data(const ZM_BLOCK  * block,
                               const void      * data,
                               uint64_t        * len);

#ifdef __cplusplus
}
#endif

#endif /* _ZM_BLOCK_H__ */
/* End. */
