/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#ifndef _ZM_EMAIL_ADDRESS_H__
#define _ZM_EMAIL_ADDRESS_H__ 1

/**
 * @file zm_email_address.h
 * @brief Email address
 */

#include "zm_data_buffer.h"
#include "zm_email_address_defs.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
  @struct _zm_email_address zm_email_address.h <zm_email_address.h>
  @brief Email address record
*/
struct ZM_API _zm_email_address
{
	/** Group            */
	ZM_DATA_BUFFER      group;
	/** Name             */
	ZM_DATA_BUFFER      name;
	/** Email route      */
	ZM_DATA_BUFFER      route;
	/** Username         */
	ZM_DATA_BUFFER      username;
	/** Domain           */
	ZM_DATA_BUFFER      domain;
	/** Next address     */
	ZM_EMAIL_ADDRESS  * next;

	/** Opaque data allocator */
	ZM_ALLOCATOR      * alloc;

	/* String flags           */
	uint32_t            flags;
};


/**
 * @brief Create email address object
 * @param email_address - If email_address is a NULL pointer, the function allocates memory for new object. Otherwise, the object is initialized and the address of the object is returned.
 * @param buffer - Source data buffer object
 * @return An initialized ZM_EMAIL_ADDRESS * object. NULL if there was insufficient memory to allocate a new object.
 */
ZM_EMAIL_ADDRESS * zm_email_address_create(ZM_EMAIL_ADDRESS  * email_address,
                                           ZM_DATA_BUFFER    * buffer);

/**
 * @brief Reset email address object
 * @param email_address - Object to reset
 */
void zm_email_address_reset(ZM_EMAIL_ADDRESS * email_address);

/**
 * @brief Destroy email address object
 * @param email_address - Object to destroy
 */
void zm_email_address_destroy(ZM_EMAIL_ADDRESS  * email_address);

/*
 * Destroy email address chain; iterate tthrough chain and free all resources
 * @param email_address - First object in chain to destroy
 */
void zm_email_address_destroy_chain(ZM_EMAIL_ADDRESS  * email_address);

/**
 * @brief Get next email address or NULL, if no next record available
 * @param email_address - Email address object
 * @return Next multipart subpart or NULL if no subpart/wrong part type
 */
ZM_EMAIL_ADDRESS * zm_email_address_next(const ZM_EMAIL_ADDRESS  * email_address);

#ifdef __cplusplus
}
#endif

#endif /* _ZERO_MIME_EMAIL_ADDRESS_H__ */
/* End. */
