/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#ifndef _ZM_ENVELOPE_H__
#define _ZM_ENVELOPE_H__ 1

/**
 * @file zm_envelope.h
 * @brief Envelope
 */
#include "zm_data_buffer.h"
#include "zm_envelope_defs.h"
#include "zm_email_address.h"
#include "zm_mem_defs.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
  @struct _zm_envelope zm_envelope.h <zm_envelope.h>
  @brief Message envelope
*/
struct ZM_API _zm_envelope
{
	/** Date                  */
	ZM_DATA_BUFFER      date;
	/** Date, unix timestamp  */
	int64_t             date_ts;
	/** Timezone              */
	int64_t             date_ts_tz;
	/** Subject               */
	ZM_DATA_BUFFER      subject;
	/** From field            */
	ZM_EMAIL_ADDRESS  * from;
	/** Sender field          */
	ZM_EMAIL_ADDRESS  * sender;
	/** Reply to field        */
	ZM_EMAIL_ADDRESS  * reply_to;
	/** To field              */
	ZM_EMAIL_ADDRESS  * to;
	/** Cc field              */
	ZM_EMAIL_ADDRESS  * cc;
	/** Bcc field            */
	ZM_EMAIL_ADDRESS  * bcc;
	/** In-Reply-To field     */
	ZM_DATA_BUFFER      in_reply_to;
	/** Message-Id field      */
	ZM_DATA_BUFFER      message_id;

	/** Opaque data allocator */
	ZM_ALLOCATOR      * alloc;

	/* String flags           */
	uint32_t            flags;
};

/**
 * @brief Create envelope object
 * @param envelope - If envelope is a NULL pointer, the function allocates memory for new object. Otherwise, the object is initialized and the address of the object is returned.
 * @param buffer - Source data buffer object
 * @return An initialized ZM_ENVELOPE * object. NULL if there was insufficient memory to allocate a new object.
 */
ZM_ENVELOPE * zm_envelope_create(ZM_ENVELOPE     * envelope,
                                 ZM_DATA_BUFFER  * buffer);

/**
 * @brief Reset envelope
 * @param envelope - Object to reset
 */
void zm_envelope_reset(ZM_ENVELOPE  * envelope);

/**
 * @brief Destroy envelope
 * @param envelope - Object to destroy
 */
void zm_envelope_destroy(ZM_ENVELOPE  * envelope);

#ifdef __cplusplus
}
#endif

#endif /* _ZM_ENVELOPE_H__ */
/* End. */
