/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#ifndef _ZM_HELPERS_H__
#define _ZM_HELPERS_H__ 1

#include "zm_iterator_defs.h"
#include "zm_mem_defs.h"
#include "zm_struct.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @brief Skip space between tokens
 */
int32_t zm_skip_space(ZM_ITERATOR        * it_start,
                      const ZM_ITERATOR  * it_end,
                      const int32_t        ignore_lf);

/**
 * @brief Parse "key: value" pairs
 */
int32_t zm_get_key_val(ZM_ITERATOR        * it_start,
                       const ZM_ITERATOR  * it_end,
                       ZM_ITERATOR        * it_key_start,
                       ZM_ITERATOR        * it_key_end,
                       ZM_ITERATOR        * it_val_start,
                       ZM_ITERATOR        * it_val_end,
                       uint64_t           * line);

/**
 * @brief Parse "x/y; foo=bar; bar=baz; baz=boo" string
 */
int32_t zm_parse_key_val(ZM_ITERATOR        * it_start,
                         const ZM_ITERATOR  * it_end,
                         ZM_BLOCK           * raw_key,
                         ZM_BLOCK           * raw_val,
                         uint32_t           * pair_num,
                         ZM_KEY_VAL        ** key_vals,
                         ZM_ALLOCATOR       * alloc,
                         const int32_t        special_case);

/**
 * @brief Compare key with given pattern
 */
int32_t zm_compare(const char      * key,
                   const uint64_t    key_len,
                   const char      * pattern);

/**
 * @brief Compare strings
 */
uint32_t zm_compare_list(const char      * key,
                         const uint64_t    key_len,
                         const char     ** keys,
                         const uint32_t  * vals);

/**
 * @brief Find substring
 */
int32_t zm_find(ZM_ITERATOR          * it_start,
                const unsigned char  * sequence,
                uint64_t               seq_len,
                uint64_t             * line);

/**
 * @brief Find boundary
 */
int32_t zm_find_boundary(ZM_ITERATOR          * it_start,
                         ZM_ITERATOR          * it_boundaty_start,
                         ZM_ITERATOR          * it_end,
                         const unsigned char  * str_boundary,
                         const uint64_t         boundary_len,
                         uint64_t             * line,
                         const int32_t          skip_crlf);

#ifdef __cplusplus
}
#endif

#endif /* _ZM_HELPERS_H__ */
/* End. */
