/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#ifndef _ZM_PART_HANDLER_H__
#define _ZM_PART_HANDLER_H__ 1

/**
  @file zm_part_handler.h
  @brief Zero MIME part data extractor handler
*/
#include "zm_address_parser_defs.h"
#include "zm_part_handler_defs.h"
#include "zm_envelope.h"
#include "zm_iconv_defs.h"
#include "zm_rbtree.h"
#include "zm_struct_defs.h"
#include "zm_time_parser_defs.h"

#define ZM_EXTRACT_ENVELOPE 0x00000001
#define ZM_RECODE_TEXT_PART 0x00000002

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @struct _zm_part_handler zm_part_handler.h <zm_part_handler.h>
 * @brief MIME part handler
 */
struct ZM_API _zm_part_handler
{
	/** Envelope                       */
	ZM_ENVELOPE          envelope;
	/** Decoded envelope               */
	ZM_ENVELOPE          decoded_envelope;
	/** Part data                      */
	ZM_DATA_BUFFER       part_data;
	/** Part data, buffer for recoding */
	ZM_DATA_BUFFER       part_data_recoded;
	/** Map of handlers                */
	ZM_RB_TREE           handlers;
	/** Text converter                 */
	ZM_ICONV           * iconv;
	/** Address parser                 */
	ZM_ADDRESS_PARSER  * address_parser;
	/** Date/time parser               */
	ZM_ABBREV          * abbrev;
	/** Opaque data allocator          */
	ZM_ALLOCATOR       * alloc;
	/* Flags                           */
	uint32_t             flags;
};

/**
 * @brief Create MIME part handler
 */
ZM_PART_HANDLER * zm_part_handler_create(ZM_PART_HANDLER    * part_handler,
                                         ZM_ICONV           * iconv,
                                         ZM_ABBREV          * abbrev,
                                         ZM_ADDRESS_PARSER  * address_parser,
                                         ZM_DATA_BUFFER     * buffer);

/**
 * @brief Destroy MIME part handler
 */
void zm_part_handler_destroy(ZM_PART_HANDLER  * part_handler);

/**
 * @brief Destroy MIME part handler
 */
int32_t zm_part_handler_register_handler(ZM_PART_HANDLER          * part_handler,
                                         enum ZM_CONTENT_TYPE       content_type,
                                         enum ZM_CONTENT_SUBTYPE    content_subtype,
                                         void                     * opaque,
                                         zm_start_part_fn           start_part_fn,
                                         zm_add_part_data_fn        add_part_data_fn,
                                         zm_end_part_fn             end_part_fn);

/**
 * @brief Decode MIME-encoded header
 */
int32_t zm_part_handler_decode_mime_encoded(ZM_PART_HANDLER  * part_handler,
                                            const void       * data,
                                            const ZM_BLOCK   * block,
                                            ZM_DATA_BUFFER   * dst);

/**
 * @brief Extract data
 */
int32_t zm_part_handler_extract_data(ZM_PART_HANDLER  * part_handler,
                                     const void       * data,
                                     const uint64_t     data_len,
                                     ZM_PART          * part,
                                     const uint32_t     flags);

#ifdef __cplusplus
}
#endif

#endif /** _ZM_PART_HANDLER_H__ */
/** End. */
