/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#ifndef _ZM_PART_HANDLER_DEFS_H__
#define _ZM_PART_HANDLER_DEFS_H__ 1

/**
  @file zm_part_handler_defs.h
  @brief Zero MIME part data extractor handler, type definitions
*/
#include "zm_data_buffer_defs.h"
#include "zm_envelope_defs.h"
#include "zm_struct_defs.h"

#define ZM_PART_OK             0x0001
#define ZM_PART_ERROR          0x0020
#define ZM_PART_NEED_DATA      0x0040
#define ZM_PART_NEED_SUBPARTS  0x0080
#define ZM_PART_BREAK          0x0100
#define ZM_PART_RAW_DATA       0x0200

#ifdef __cplusplus
extern "C"
{
#endif

/** Start of MIME part */
typedef int32_t (*zm_start_part_fn)(void                  * opaque,
                                    const void            * data,
                                    const ZM_PART         * part,
                                    const ZM_DATA_BUFFER  * part_name,
                                    const ZM_ENVELOPE     * envelope,
                                    const ZM_ENVELOPE     * decoded_envelope);

/** Add MIME part data */
typedef int32_t (*zm_add_part_data_fn)(void                  * opaque,
                                       const void            * data,
                                       const ZM_PART         * part,
                                       const ZM_DATA_BUFFER  * part_name,
                                       const void            * part_data,
                                       const uint64_t          part_data_len);

/** End of MIME part */
typedef int32_t (*zm_end_part_fn)(void                  * opaque,
                                  const void            * data,
                                  const ZM_PART         * part,
                                  const ZM_DATA_BUFFER  * part_name);

/** Data buffer */
typedef struct _zm_part_handler    ZM_PART_HANDLER;

#ifdef __cplusplus
}
#endif

#endif /** _ZM_PART_HANDLER_DEFS_H__ */
/** End. */
