/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#ifndef _ZM_RBTREE_CMP_H__
#define _ZM_RBTREE_CMP_H__ 1

/**
 * @file zm_rbtree_cmp.h
 * @brief Zero MIME red-black tree comparatros
 */

#include "zm_params.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @brief Compare(x,y) should return 1 if *a > *b, -1 if *a < *b, and 0 otherwise
 * @param opaque - Opaque data
 * @param x - First value to compare
 * @param y - Second value to compare
 * @return eturn 1 if *a > *b, -1 if *a < *b, and 0 otherwise
 */
int zm_rb_void_cmp(void        * opaque,
                   const void  * x,
                   const void  * y);

/**
 * @brief Compare(x,y) should return 1 if *a > *b, -1 if *a < *b, and 0 otherwise
 * @param opaque - Opaque data
 * @param x - First value to compare
 * @param y - Second value to compare
 * @return eturn 1 if *a > *b, -1 if *a < *b, and 0 otherwise
 */
int zm_rb_string_cmp(void        * opaque,
                     const void  * x,
                     const void  * y);

/**
 * @brief Compare(x,y) should return 1 if *a > *b, -1 if *a < *b, and 0 otherwise
 * @param opaque - Opaque data
 * @param x - First value to compare
 * @param y - Second value to compare
 * @return eturn 1 if *a > *b, -1 if *a < *b, and 0 otherwise
 */
int zm_rb_string_case_cmp(void        * opaque,
                          const void  * x,
                          const void  * y);

/**
 * @brief Compare(x,y) should return 1 if *a > *b, -1 if *a < *b, and 0 otherwise
 * @param opaque - Opaque data
 * @param x - First value to compare
 * @param y - Second value to compare
 * @return eturn 1 if *a > *b, -1 if *a < *b, and 0 otherwise
 */
int zm_rb_int32_cmp(void        * opaque,
                    const void  * x,
                    const void  * y);

/**
 * @brief Compare(x,y) should return 1 if *a > *b, -1 if *a < *b, and 0 otherwise
 * @param opaque - Opaque data
 * @param x - First value to compare
 * @param y - Second value to compare
 * @return eturn 1 if *a > *b, -1 if *a < *b, and 0 otherwise
 */
int zm_rb_uint32_cmp(void        * opaque,
                     const void  * x,
                     const void  * y);

/**
 * @brief Compare(x,y) should return 1 if *a > *b, -1 if *a < *b, and 0 otherwise
 * @param opaque - Opaque data
 * @param x - First value to compare
 * @param y - Second value to compare
 * @return eturn 1 if *a > *b, -1 if *a < *b, and 0 otherwise
 */
int zm_rb_int64_cmp(void        * opaque,
                    const void  * x,
                    const void  * y);

/**
 * @brief Compare(x,y) should return 1 if *a > *b, -1 if *a < *b, and 0 otherwise
 * @param opaque - Opaque data
 * @param x - First value to compare
 * @param y - Second value to compare
 * @return eturn 1 if *a > *b, -1 if *a < *b, and 0 otherwise
 */
int zm_rb_uint64_cmp(void        * opaque,
                     const void  * x,
                     const void  * y);

/**
 * @brief Compare(x,y) should return 1 if *a > *b, -1 if *a < *b, and 0 otherwise
 * @param opaque - Opaque data
 * @param x - First value to compare
 * @param y - Second value to compare
 * @return eturn 1 if *a > *b, -1 if *a < *b, and 0 otherwise
 */
int zm_rb_pointer_cmp(void        * opaque,
                      const void  * x,
                      const void  * y);

#ifdef __cplusplus
}
#endif

#endif /* _ZM_RBTREE_CMP_H__ */
/* End */
