/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#ifndef _ZM_RBTREE_FN_H__
#define _ZM_RBTREE_FN_H__ 1

/**
 * @file zm_rbtree_fn.h
 * @brief Zero MIME red-black tree freeing functions
 */

#include "zm_params.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @brief Do not destroy key and value
 * @param opaque - Not used
 * @param key - Not used
 * @param value - Not used
 */
void zm_rb_destroy_null(void  * opaque,
                        void  * key,
                        void  * val);

/**
 * @brief Destroy only key
 * @param opaque - Opaque data, pointer to ZM_ALLOCATOR object
 * @param key - Memory to free
 * @param value - Not used
 */
void zm_rb_destroy_key(void  * opaque,
                       void  * key,
                       void  * val);

/**
 * @brief Destroy only value
 * @param opaque - Opaque data, pointer to ZM_ALLOCATOR object
 * @param key - Not used
 * @param value - Memory to free
 */
void zm_rb_destroy_val(void  * opaque,
                       void  * key,
                       void  * val);

/**
 * @brief Destroy key and value
 * @param opaque - Opaque data, pointer to ZM_ALLOCATOR object
 * @param key - Memory to free
 * @param value - Memory to free
 */
void zm_rb_destroy_key_val(void  * opaque,
                           void  * key,
                           void  * val);

#ifdef __cplusplus
}
#endif

#endif /* _ZM_RBTREE_FN_H__ */
/* End */
