/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#ifndef _ZM_STRUCT_DEFS_H__
#define _ZM_STRUCT_DEFS_H__ 1

/**
  @file zm_struct_defs.h
  @brief Zero MIME data structures, type definitions
*/

#include "zm_params.h"

#ifdef __cplusplus
extern "C"
{
#endif

typedef struct _zm_key_val       ZM_KEY_VAL;
typedef struct _zm_field_val     ZM_FIELD_VAL;
typedef struct _zm_field_rec     ZM_FIELD_REC;
typedef struct _zm_header        ZM_HEADER;
typedef struct _zm_part_data     ZM_PART_DATA;

typedef struct _zm_basic_part    ZM_BASIC_PART;
typedef struct _zm_part          ZM_PART;
typedef struct _zm_base_part     ZM_BASE_PART;
typedef struct _zm_text_part     ZM_TEXT_PART;
typedef struct _zm_multi_part    ZM_MULTI_PART;
typedef struct _zm_message_part  ZM_MESSAGE_PART;

/**
 * @brief Content type
 */
enum ZM_CONTENT_TYPE
{
	TEXT        = 0x0001,
	BINARY_PART = 0x0002,
	IMAGE       = 0x0003,
	AUDIO       = 0x0004,
	VIDEO       = 0x0005,
	APPLICATION = 0x0006,
	MESSAGE     = 0x0007,
	MULTIPART   = 0x0008,
	OTHER_TYPE  = 0x0000
};

/**
 * @brief Content subtype
 */
enum ZM_CONTENT_SUBTYPE
{
	/* Plain subtypes */
	PLAIN  = 0x0001, HTML = 0x0002, RFC822_HEADERS = 0x0003, DIRECTORY  = 0x0004, RTF  = 0x0005, XML = 0x0006, CALENDAR = 0x0007,
	/* Image subtypes */
	GIF    = 0x0011, JPEG = 0x0012, PNG            = 0x0013, SVG        = 0x0014, TIFF = 0x0015,
	/* Audio subtypes */
	BASIC  = 0x0021, MIDI = 0x0022, MP4            = 0x0023, OGG        = 0x0024,
	/* Video subtypes */
	MPEG   = 0x0031,
	/* Application subtypes */
	GZIP   = 0x0041, OCTET_STREAM = 0x0042, PDF           = 0x0043, X_RAR                    = 0x0044, X_TAR           = 0x0045, ZIP             = 0x0046,
	/* Message subtypes */
	RFC822 = 0x0051, PARTIAL      = 0x0052, EXTERNAL_BODY = 0x0053, DISPOSITION_NOTIFICATION = 0x0054, DELIVERY_STATUS = 0x0055, FEEDBACK_REPORT = 0x0056,
	/* Multipart subtypes */
	MIXED  = 0x0061, RELATED      = 0x0062, ALTERNATIVE   = 0x0063, PARALLEL                 = 0x0064, DIGEST          = 0x0065, REPORT          = 0x0066, SIGNED = 0x0067, ENCRYPTED = 0x0068, APPLEDOUBLE = 0x0069,
	/* Other subtype */
	OTHER_SUBTYPE = 0x0000
};

/**
 * @brief Charset
 */
enum ZM_CHARSET
{
	/* Default type */
	US_ASCII = 1,

	/* Other charset, for example - sindarin or quenya */
	OTHER_CHARSET = 2,

	/* UTF, for normal people */
	UTF_7 = 7, UTF_8 = 8, UTF_16 = 16,

	/* KOI, special case for brain-damaged russian old farts */
	KOI_8R = 101, KOI_8U = 102,

	/* Windows old farts */
	WINDOWS_1250 = 1250,  WINDOWS_1251 = 1251,  WINDOWS_1252 = 1252,  WINDOWS_1253 = 1253,
	WINDOWS_1254 = 1254,  WINDOWS_1255 = 1255,  WINDOWS_1256 = 1256,  WINDOWS_1257 = 1257,
	WINDOWS_1258 = 1258,

	/* ISO-8859-X enterprise old farts */
	ISO8859_1    = 10001, ISO8859_2    = 10002, ISO8859_3    = 10003, ISO8859_4    = 10004,
	ISO8859_5    = 10005, ISO8859_6    = 10006, ISO8859_7    = 10007, ISO8859_8    = 10008,
	ISO8859_9    = 10009, ISO8859_10   = 10010, ISO8859_11   = 10011, ISO8859_13   = 10013,
	ISO8859_14   = 10014, ISO8859_15   = 10015, ISO8859_16   = 10016,

	/* CP-XXX, DOS fossilized dinosaurs */
	CP_437       = 10437, CP_737       = 10737, CP_775       = 10775, CP_850       = 10850,
	CP_852       = 10852, CP_853       = 10853, CP_855       = 10855, CP_857       = 10857,
	CP_858       = 10858, CP_860       = 10860, CP_861       = 10861, CP_862       = 10862,
	CP_863       = 10863, CP_864       = 10864, CP_865       = 10865, CP_866       = 10866,
	CP_869       = 10869, CP_874       = 10874, CP_936       = 10936, CP_950       = 10950,
	CP_1125      = 11125,

	/* Eastern old farts */
	TIS = 100100, SHIFT_JIS = 100200, BIG5 = 100300, EUC_KR = 100400
};

/**
 * @brief Content transfer encoding
 */
enum ZM_CONTENT_TRANSFER_ENCODING
{
	/** 7-bit, us-ascii */
	BIT7             = 0x01,
	/** 8-bit, usually - utf-8 */
	BIT8             = 0x02,
	/** Binary */
	BINARY           = 0x03,
	/** Quoted-printable */
	QUOTED_PRINTABLE = 0x04,
	/** Base64 */
	BASE64           = 0x05,
	/** Uuencode */
	UU_ENCODED       = 0x06,
	/** Other encoding */
	OTHER_ENCODING   = 0x00
};

/**
 * @brief Content disposition
 */
enum ZM_CONTENT_DISPOSITION
{
	INLINE     = 0x00,
	ATTACHMENT = 0x01
};

#ifdef __cplusplus
}
#endif

#endif /* _ZM_STRUCT_DEFS_H__ */
/* End. */
