/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#ifndef _ZM_TIME_PARSER_H__
#define _ZM_TIME_PARSER_H__ 1

/**
  @file zm_time_parser.h
  @brief Zero MIME datetime parser
*/

#include "zm_mem_defs.h"
#include "zm_time_parser_defs.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @brief Load custom timezone abbrevations
 * @param file - File to load
 * @param alloc - Memory allocator
 * @param abbrev - List of abbrevations [out]
 * @return 0 - if success, -1 - otherwise
 */
int32_t zm_load_abbrev(const char     * file,
                       ZM_ALLOCATOR   * alloc,
                       ZM_ABBREV     ** abbrev);

/**
 * @brief Print custom timezone abbrevations
 * @param abbrev - List of abbrevations
 */
void zm_print_abbrev(ZM_ABBREV  * abbrev);

/**
 * @brief Free timezone abbrevations
 * @param abbrev - List of abbrevations
 */
void zm_free_abbrev(ZM_ABBREV  * abbrev);

/**
 * @brief Parse datetime string
 * @param dt - Datetime string
 * @param dt_len - Datetime length
 * @param ts - Un*x timestamp, seconds [out]
 * @param ts_offset - Timezone offset, seconds [out]
 * @param abbrev - List of abbrevations
 * @return 0 - if success, -1 - otherwise
 */
int32_t zm_parse_datetime(const char      * dt,
                          const uint32_t    dt_len,
                          int64_t         * ts,
                          int64_t         * ts_offset,
                          ZM_ABBREV       * abbrev);

#ifdef __cplusplus
}
#endif

#endif /* _ZM_TIME_PARSER_H__ */
/* End. */
