/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#include "zm_email_address.h"
#include "zm_mem.h"

#include <stdio.h>

/*
 * Create email address object
 */
ZM_EMAIL_ADDRESS * zm_email_address_create(ZM_EMAIL_ADDRESS  * email_address,
                                           ZM_DATA_BUFFER    * buffer)
{
	uint32_t        flags = 0x0000;
	ZM_ALLOCATOR  * alloc = zm_data_buffer_get_allocator(buffer);

	if (email_address == NULL)
	{
		email_address = (ZM_EMAIL_ADDRESS *)zm_allocate(alloc, sizeof(ZM_EMAIL_ADDRESS));
		flags = 0x0001;
	}

	if (email_address == NULL) { return NULL; }

	if (zm_data_buffer_clone(&(email_address -> group),    buffer) != NULL) { flags |= 0x0010; }
	if (zm_data_buffer_clone(&(email_address -> name),     buffer) != NULL) { flags |= 0x0020; }
	if (zm_data_buffer_clone(&(email_address -> route),    buffer) != NULL) { flags |= 0x0040; }
	if (zm_data_buffer_clone(&(email_address -> username), buffer) != NULL) { flags |= 0x0080; }
	if (zm_data_buffer_clone(&(email_address -> domain),   buffer) != NULL) { flags |= 0x0100; }

	email_address -> next  = NULL;
	email_address -> alloc = alloc;
	email_address -> flags = flags;

	if ((flags & 0x01F0) != 0x01F0)
	{
		zm_email_address_destroy(email_address);
		return NULL;
	}

return email_address;
}

/*
 * Reset email address
 */
void zm_email_address_reset(ZM_EMAIL_ADDRESS * email_address)
{
	zm_data_buffer_reset(&(email_address -> group));
	zm_data_buffer_reset(&(email_address -> name));
	zm_data_buffer_reset(&(email_address -> route));
	zm_data_buffer_reset(&(email_address -> username));
	zm_data_buffer_reset(&(email_address -> domain));

	zm_email_address_destroy_chain(email_address -> next);

	email_address -> next = NULL;
}

/*
 * Destroy email address object
 */
void zm_email_address_destroy(ZM_EMAIL_ADDRESS  * email_address)
{
	ZM_ALLOCATOR  * alloc = email_address -> alloc;

	if ((email_address -> flags) & 0x0010) { zm_data_buffer_destroy(&(email_address -> group));    }
	if ((email_address -> flags) & 0x0020) { zm_data_buffer_destroy(&(email_address -> name));     }
	if ((email_address -> flags) & 0x0040) { zm_data_buffer_destroy(&(email_address -> route));    }
	if ((email_address -> flags) & 0x0080) { zm_data_buffer_destroy(&(email_address -> username)); }
	if ((email_address -> flags) & 0x0100) { zm_data_buffer_destroy(&(email_address -> domain));   }

	if ((email_address -> flags) & 0x0001)
	{
		zm_deallocate(alloc, email_address);
	}
}

/*
 * Destroy email address chain
 */
void zm_email_address_destroy_chain(ZM_EMAIL_ADDRESS  * email_address)
{
	while (email_address != NULL)
	{
		ZM_EMAIL_ADDRESS  * next = email_address -> next;

		zm_email_address_destroy(email_address);

		email_address = next;
	}
}

/*
 * Get next email address or NULL, if no next record available
 */
ZM_EMAIL_ADDRESS * zm_email_address_next(const ZM_EMAIL_ADDRESS  * email_address)
{
	if (email_address == NULL) { return NULL; }

return email_address -> next;
}

/* End. */
