/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#include "zm_envelope.h"
#include "zm_mem.h"

/*
 * Create envelope
 */
ZM_ENVELOPE * zm_envelope_create(ZM_ENVELOPE     * envelope,
                                 ZM_DATA_BUFFER  * buffer)
{
	uint32_t        flags = 0x0000;
	ZM_ALLOCATOR  * alloc = envelope -> alloc;

	if (envelope == NULL)
	{
		envelope = (ZM_ENVELOPE *)zm_allocate(alloc, sizeof(ZM_ENVELOPE));
		flags = 0x0001;
	}

	if (envelope == NULL) { return NULL; }

	envelope -> date_ts    = -1;
	envelope -> date_ts_tz =  0;

	envelope -> from     = NULL;
	envelope -> sender   = NULL;
	envelope -> reply_to = NULL;
	envelope -> to       = NULL;
	envelope -> cc       = NULL;
	envelope -> bcc      = NULL;

	if (zm_data_buffer_clone(&(envelope -> date),        buffer) != NULL) { flags |= 0x0010; }
	if (zm_data_buffer_clone(&(envelope -> subject),     buffer) != NULL) { flags |= 0x0020; }
	if (zm_data_buffer_clone(&(envelope -> in_reply_to), buffer) != NULL) { flags |= 0x0040; }
	if (zm_data_buffer_clone(&(envelope -> message_id),  buffer) != NULL) { flags |= 0x0080; }

	envelope -> alloc = alloc;
	envelope -> flags = flags;

	if ((flags & 0x00F0) != 0x00F0)
	{
		zm_envelope_destroy(envelope);
		return NULL;
	}

return envelope;
}

/*
 * Reset envelope
 */
void zm_envelope_reset(ZM_ENVELOPE  * envelope)
{
	envelope -> date_ts    = -1;
	envelope -> date_ts_tz =  0;

	zm_data_buffer_reset(&(envelope -> date));
	zm_data_buffer_reset(&(envelope -> subject));
	zm_data_buffer_reset(&(envelope -> in_reply_to));
	zm_data_buffer_reset(&(envelope -> message_id));

	zm_email_address_destroy_chain(envelope -> from);
	zm_email_address_destroy_chain(envelope -> sender);
	zm_email_address_destroy_chain(envelope -> reply_to);
	zm_email_address_destroy_chain(envelope -> to);
	zm_email_address_destroy_chain(envelope -> cc);
	zm_email_address_destroy_chain(envelope -> bcc);

	envelope -> from     = NULL;
	envelope -> sender   = NULL;
	envelope -> reply_to = NULL;
	envelope -> to       = NULL;
	envelope -> cc       = NULL;
	envelope -> bcc      = NULL;
}

/*
 * Destroy envelope
 */
void zm_envelope_destroy(ZM_ENVELOPE  * envelope)
{
	ZM_ALLOCATOR  * alloc = envelope -> alloc;

	if (envelope -> flags & 0x0010) { zm_data_buffer_destroy(&(envelope -> date));        }
	if (envelope -> flags & 0x0020) { zm_data_buffer_destroy(&(envelope -> subject));     }
	if (envelope -> flags & 0x0040) { zm_data_buffer_destroy(&(envelope -> in_reply_to)); }
	if (envelope -> flags & 0x0080) { zm_data_buffer_destroy(&(envelope -> message_id));  }

	zm_email_address_destroy_chain(envelope -> from);
	zm_email_address_destroy_chain(envelope -> sender);
	zm_email_address_destroy_chain(envelope -> reply_to);
	zm_email_address_destroy_chain(envelope -> to);
	zm_email_address_destroy_chain(envelope -> cc);
	zm_email_address_destroy_chain(envelope -> bcc);

	if (envelope -> flags & 0x0001)
	{
		zm_deallocate(alloc, envelope);
	}
}
/* End. */
