/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#include "zm_iterator.h"

/*
 * Create iterator
 */
void zm_iterator_create(ZM_ITERATOR  * it,
                        const void   * data,
                        uint64_t       pos)
{
	it -> data         = data;
	it -> pos          = pos;
#ifdef ZM_EXTRA_STREAM_INFO
	it -> bits         = 0;
	it -> line_len     = 0;
	it -> max_line_len = 0;
#endif
}

/*
 * Create empty iterator
 */
void zm_iterator_zero(ZM_ITERATOR  * it)
{
	zm_iterator_create(it, NULL, 0);
}

/*
 * Copy iterator
 */
void zm_iterator_copy(ZM_ITERATOR        * it_dst,
                      const ZM_ITERATOR  * it_src)
{
	it_dst -> data = it_src -> data;
	it_dst -> pos  = it_src -> pos;

#ifdef ZM_EXTRA_STREAM_INFO
	it_dst -> bits         = it_src -> bits;
	it_dst -> line_len     = it_src -> line_len;
	it_dst -> max_line_len = it_src -> max_line_len;
#endif
}

/*
 * Compare iterators
 */
int32_t zm_iterator_equal(const ZM_ITERATOR  * it_x,
                          const ZM_ITERATOR  * it_y)
{
	unsigned char * x1 = (unsigned char *)(it_x -> data) + it_x -> pos;
	unsigned char * y1 = (unsigned char *)(it_y -> data) + it_y -> pos;

return (x1 == y1) ? 1 : 0;
}

/*
 * Compare iterators
 */
int32_t zm_iterator_not_equal(const ZM_ITERATOR  * it_x,
                              const ZM_ITERATOR  * it_y)
{
	;;
return !zm_iterator_equal(it_x, it_y);
}

/*
 * Get current position
 */
uint64_t zm_iterator_get_pos(const ZM_ITERATOR  * it)
{
	;;
return it -> pos;
}

/*
 * Get disatnce
 */
int64_t zm_iterator_get_distance(const ZM_ITERATOR  * it_start,
                                 const ZM_ITERATOR  * it_end)
{
	unsigned char * uc_start = (unsigned char *)(it_start -> data) + it_start -> pos;
	unsigned char * uc_end   = (unsigned char *)(it_end -> data) + it_end -> pos;

return uc_end - uc_start;
}

/*
 * Get current address
 */
const void * zm_iterator_get_addr(const ZM_ITERATOR  * it)
{
	;;
return (unsigned char *)(it -> data) + it -> pos;
}

/*
 * Get data between two iterators
 */
const void * zm_iterator_get_data(const ZM_ITERATOR  * it_start,
                                  const ZM_ITERATOR  * it_end,
                                  uint64_t           * len)
{
	unsigned char * uc_start = (unsigned char *)(it_start -> data) + it_start -> pos;
	unsigned char * uc_end   = (unsigned char *)(it_end -> data) + it_end -> pos;

	*len = uc_end - uc_start;

return uc_start;
}

/*
 * Iterate next symbol
 */
void zm_iterator_next(ZM_ITERATOR  * it)
{
	it -> pos += 1;
}

/*
 * Get current address
 */
const unsigned char zm_iterator_ref(const ZM_ITERATOR  * it)
{
	const unsigned char uc = *((unsigned char *)zm_iterator_get_addr(it));

#ifdef ZM_EXTRA_STREAM_INFO
	if ((uc < ' ' && uc != 0x09 && uc != 0x0A && uc != 0x0D) || uc > 0x7F) { ((ZM_ITERATOR *)it) -> bits |= 0x80; }

	if (uc != 0x0A) { ((ZM_ITERATOR *)it) -> line_len += 1; }
	else
	{
		if (it -> max_line_len < it -> line_len) { ((ZM_ITERATOR *)it) -> max_line_len = it -> line_len; }
		((ZM_ITERATOR *)it) -> line_len = 0;
	}
#endif
return uc;
}

/*
 * Get stream properties
 */
void zm_iterator_stream_props(ZM_ITERATOR   * it,
                              unsigned char * bits,
                              uint64_t      * line_len)
{
#ifdef ZM_EXTRA_STREAM_INFO
	*bits     = it -> bits;
	*line_len = it -> max_line_len;
#else
	*bits     = 0;
	*line_len = 0;
#endif
}
/* End. */
