/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#include "zm_mem.h"

#include <stdlib.h>
#include <string.h>

/*
 * Allocate data from pool and fill memory block with zeroes
 */
void * zm_calloc(void    * opaque,
                 size_t    size)
{
	void * data = malloc(size);
	if (data == NULL) { return NULL; }

	memset(data, 0, size);

return data;
}

/*
 * Reallocate data from pool
 */
void * zm_realloc(void    * opaque,
                  void    * buf,
                  size_t    size)
{
	;;
return realloc(buf, size);
}

/*
 * Free memory block from pool
 */
void zm_dealloc(void  * opaque,
                void  * data)
{
	free(data);
}

/*
 * Allocate memory
 */
void * zm_allocate(ZM_ALLOCATOR  * alloc,
                   size_t          size)
{
	;;
return alloc -> allocate(alloc -> opaque, size);
}

/*
 * Reallocate memory
 */
void * zm_reallocate(ZM_ALLOCATOR  * alloc,
                     void          * data,
                     size_t          size)
{
	;;
return alloc -> reallocate(alloc -> opaque, data, size);
}

/*
 * Deallocate memory
 */
void zm_deallocate(ZM_ALLOCATOR  * alloc,
                   void          * data)
{
	alloc -> deallocate(alloc -> opaque, data);
}

/*
  * Duplicate block memory
  */
void * zm_duplicate(ZM_ALLOCATOR  * alloc,
                    const void    * data,
                    size_t          size)
{
	void * new_data = zm_allocate(alloc, size);
	if (new_data == NULL) { return NULL; }

	memcpy(new_data, data, size);

return new_data;
}

/*
 * Default allocator
 */
ZM_ALLOCATOR zm_default_allocator =
{
	/* Pointer           */
	NULL,
	/* Allocate memory   */
	zm_calloc,
	/* Reallocate memory */
	zm_realloc,
	/* Deallocate memory */
	zm_dealloc
};

/* End. */
