/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#include "zm_mem.h"
#include "zm_struct.h"

/*
 * Get printable name of part content type
 */
const char * zm_get_part_type(const ZM_PART  * part,
                              const void     * msg,
                              uint64_t       * type_len)
{
	switch (part -> type)
	{
		case TEXT:        *type_len = sizeof("text")        - 1; return "text";
		case BINARY_PART: *type_len = sizeof("binary")      - 1; return "binary";
		case IMAGE:       *type_len = sizeof("image")       - 1; return "image";
		case AUDIO:       *type_len = sizeof("audio")       - 1; return "audio";
		case VIDEO:       *type_len = sizeof("video")       - 1; return "video";
		case APPLICATION: *type_len = sizeof("application") - 1; return "application";
		case MESSAGE:     *type_len = sizeof("message")     - 1; return "message";
		case MULTIPART:   *type_len = sizeof("multipart")   - 1; return "multipart";
		case OTHER_TYPE:
		default:
			return (const char *)zm_block_get_data(&(part -> raw_type), msg, type_len);
	}
/* Make compiler happy */
return NULL;
}

/*
 * Get printable name of part content subtype
 */
const char * zm_get_part_subtype(const ZM_PART  * part,
                                 const void     * msg,
                                 uint64_t       * subtype_len)
{
	switch (part -> subtype)
	{
		case PLAIN:                   *subtype_len = sizeof("plain")                       - 1; return "plain";
		case HTML:                    *subtype_len = sizeof("html")                        - 1; return "html";
		case RFC822_HEADERS:          *subtype_len = sizeof("rfc822-headers")              - 1; return "rfc822-headers";
		case DIRECTORY:               *subtype_len = sizeof("directory")                   - 1; return "directory";
		case RTF:                     *subtype_len = sizeof("rtf")                         - 1; return "rtf";
		case XML:                     *subtype_len = sizeof("xml")                         - 1; return "xml";
		case CALENDAR:                *subtype_len = sizeof("calendar")                    - 1; return "calendar";
		case GIF:                     *subtype_len = sizeof("gif")                         - 1; return "gif";
		case JPEG:                    *subtype_len = sizeof("jpeg")                        - 1; return "jpeg";
		case PNG:                     *subtype_len = sizeof("png")                         - 1; return "png";
		case SVG:                     *subtype_len = sizeof("svg")                         - 1; return "svg";
		case TIFF:                    *subtype_len = sizeof("tiff")                        - 1; return "tiff";
		case BASIC:                   *subtype_len = sizeof("basic")                       - 1; return "basic";
		case MIDI:                    *subtype_len = sizeof("midi")                        - 1; return "midi";
		case MP4:                     *subtype_len = sizeof("mp4")                         - 1; return "mp4";
		case OGG:                     *subtype_len = sizeof("ogg")                         - 1; return "ogg";
		case MPEG:                    *subtype_len = sizeof("mpeg")                        - 1; return "mpeg";
		case GZIP:                    *subtype_len = sizeof("gzip")                        - 1; return "gzip";
		case OCTET_STREAM:            *subtype_len = sizeof("octet-stream")                - 1; return "octet-stream";
		case PDF:                     *subtype_len = sizeof("pdf")                         - 1; return "pdf";
		case X_RAR:                   *subtype_len = sizeof("x-rar")                       - 1; return "x-rar";
		case X_TAR:                   *subtype_len = sizeof("x-tar")                       - 1; return "x-tar";
		case ZIP:                     *subtype_len = sizeof("zip")                         - 1; return "zip";
		case RFC822:                  *subtype_len = sizeof("rfc822")                      - 1; return "rfc822";
		case PARTIAL:                 *subtype_len = sizeof("partial")                     - 1; return "partial";
		case EXTERNAL_BODY:           *subtype_len = sizeof("external-body")               - 1; return "external-body";
		case DISPOSITION_NOTIFICATION:*subtype_len = sizeof("disposition-notification-to") - 1; return "disposition-notification-to";
		case DELIVERY_STATUS:         *subtype_len = sizeof("delivery-status")             - 1; return "delivery-status";
		case FEEDBACK_REPORT:         *subtype_len = sizeof("feedback-report")             - 1; return "feedback-report";
		case MIXED:                   *subtype_len = sizeof("mixed")                       - 1; return "mixed";
		case RELATED:                 *subtype_len = sizeof("related")                     - 1; return "related";
		case ALTERNATIVE:             *subtype_len = sizeof("alternative")                 - 1; return "alternative";
		case PARALLEL:                *subtype_len = sizeof("parallel")                    - 1; return "parallel";
		case DIGEST:                  *subtype_len = sizeof("digest")                      - 1; return "digest";
		case REPORT:                  *subtype_len = sizeof("report")                      - 1; return "report";
		case SIGNED:                  *subtype_len = sizeof("signed")                      - 1; return "signed";
		case ENCRYPTED:               *subtype_len = sizeof("encrypted")                   - 1; return "encrypted";
		case APPLEDOUBLE:             *subtype_len = sizeof("appledouble")                 - 1; return "appledouble";
		case OTHER_SUBTYPE:
		default:
			return (const char *)zm_block_get_data(&(part -> raw_subtype), msg, subtype_len);
	}
/* Make compiler happy */
return NULL;
}

/*
 * Get printable name of part content transfer encoding
 */
const char * zm_get_part_content_transfer_encoding(const ZM_PART  * part,
                                                   const void     * msg,
                                                   uint64_t       * encoding_len)
{
	switch (part -> content_transfer_encoding)
	{
		case BIT7:             *encoding_len = sizeof("7bit")             - 1; return "7bit";
		case BIT8:             *encoding_len = sizeof("8bit")             - 1; return "8bit";
		case BINARY:           *encoding_len = sizeof("binary")           - 1; return "binary";
		case QUOTED_PRINTABLE: *encoding_len = sizeof("quoted-printable") - 1; return "quoted-printable";
		case BASE64:           *encoding_len = sizeof("base64")           - 1; return "base64";
		case UU_ENCODED:       *encoding_len = sizeof("x-uuencode")       - 1; return "x-uuencode";
		case OTHER_ENCODING:
		default:
			return (const char *)zm_block_get_data(&(part -> raw_content_transfer_encoding), msg, encoding_len);
	}
/* Make compiler happy */
return NULL;
}

/*
 * Get printable name of part content disposition
 */
const char * zm_get_part_content_disposition(const ZM_PART  * part,
                                             const void     * msg,
                                             uint64_t       * disposition_len)
{
	switch (part -> content_disposition)
	{
		case INLINE:     *disposition_len = sizeof("inline")     - 1; return "inline";
		case ATTACHMENT: *disposition_len = sizeof("attachment") - 1; return "attachment";
	}

	*disposition_len = 0;

return NULL;
}

/*
 * Get printable name of character set
 */
const char * zm_get_text_part_charset(const ZM_TEXT_PART  * part,
                                      const void          * msg,
                                      uint64_t            * charset_len)
{
	switch (part -> charset)
	{
		case US_ASCII:      *charset_len = sizeof("us-ascii")     - 1; return "us-ascii";
		case UTF_7:         *charset_len = sizeof("utf-7")        - 1; return "utf-7";
		case UTF_8:         *charset_len = sizeof("utf-8")        - 1; return "utf-8";
		case UTF_16:        *charset_len = sizeof("utf-16")       - 1; return "utf-16";
		case KOI_8R:        *charset_len = sizeof("koi8-r")       - 1; return "koi8-r";
		case KOI_8U:        *charset_len = sizeof("koi8-u")       - 1; return "koi8-u";
		case WINDOWS_1250:  *charset_len = sizeof("windows-1250") - 1; return "windows-1250";
		case WINDOWS_1251:  *charset_len = sizeof("windows-1251") - 1; return "windows-1251";
		case WINDOWS_1252:  *charset_len = sizeof("windows-1252") - 1; return "windows-1252";
		case WINDOWS_1253:  *charset_len = sizeof("windows-1253") - 1; return "windows-1253";
		case WINDOWS_1254:  *charset_len = sizeof("windows-1254") - 1; return "windows-1254";
		case WINDOWS_1255:  *charset_len = sizeof("windows-1255") - 1; return "windows-1255";
		case WINDOWS_1256:  *charset_len = sizeof("windows-1256") - 1; return "windows-1256";
		case WINDOWS_1257:  *charset_len = sizeof("windows-1257") - 1; return "windows-1257";
		case WINDOWS_1258:  *charset_len = sizeof("windows-1258") - 1; return "windows-1258";
		case ISO8859_1:     *charset_len = sizeof("iso-8859-1")   - 1; return "iso-8859-1";
		case ISO8859_2:     *charset_len = sizeof("iso-8859-2")   - 1; return "iso-8859-2";
		case ISO8859_3:     *charset_len = sizeof("iso-8859-3")   - 1; return "iso-8859-3";
		case ISO8859_4:     *charset_len = sizeof("iso-8859-4")   - 1; return "iso-8859-4";
		case ISO8859_5:     *charset_len = sizeof("iso-8859-5")   - 1; return "iso-8859-5";
		case ISO8859_6:     *charset_len = sizeof("iso-8859-6")   - 1; return "iso-8859-6";
		case ISO8859_7:     *charset_len = sizeof("iso-8859-7")   - 1; return "iso-8859-7";
		case ISO8859_8:     *charset_len = sizeof("iso-8859-8")   - 1; return "iso-8859-8";
		case ISO8859_9:     *charset_len = sizeof("iso-8859-9")   - 1; return "iso-8859-9";
		case ISO8859_10:    *charset_len = sizeof("iso-8859-10")  - 1; return "iso-8859-10";
		case ISO8859_11:    *charset_len = sizeof("iso-8859-11")  - 1; return "iso-8859-11";
		case ISO8859_13:    *charset_len = sizeof("iso-8859-13")  - 1; return "iso-8859-13";
		case ISO8859_14:    *charset_len = sizeof("iso-8859-14")  - 1; return "iso-8859-14";
		case ISO8859_15:    *charset_len = sizeof("iso-8859-15")  - 1; return "iso-8859-15";
		case ISO8859_16:    *charset_len = sizeof("iso-8859-16")  - 1; return "iso-8859-16";
		case CP_437:        *charset_len = sizeof("cp437")        - 1; return "cp437";
		case CP_737:        *charset_len = sizeof("cp737")        - 1; return "cp437";
		case CP_775:        *charset_len = sizeof("cp775")        - 1; return "cp437";
		case CP_850:        *charset_len = sizeof("cp850")        - 1; return "cp437";
		case CP_852:        *charset_len = sizeof("cp852")        - 1; return "cp437";
		case CP_853:        *charset_len = sizeof("cp853")        - 1; return "cp437";
		case CP_855:        *charset_len = sizeof("cp855")        - 1; return "cp437";
		case CP_857:        *charset_len = sizeof("cp857")        - 1; return "cp437";
		case CP_858:        *charset_len = sizeof("cp858")        - 1; return "cp437";
		case CP_860:        *charset_len = sizeof("cp860")        - 1; return "cp437";
		case CP_861:        *charset_len = sizeof("cp861")        - 1; return "cp437";
		case CP_862:        *charset_len = sizeof("cp862")        - 1; return "cp437";
		case CP_863:        *charset_len = sizeof("cp863")        - 1; return "cp437";
		case CP_864:        *charset_len = sizeof("cp864")        - 1; return "cp437";
		case CP_865:        *charset_len = sizeof("cp865")        - 1; return "cp437";
		case CP_866:        *charset_len = sizeof("cp866")        - 1; return "cp437";
		case CP_869:        *charset_len = sizeof("cp869")        - 1; return "cp437";
		case CP_874:        *charset_len = sizeof("cp874")        - 1; return "cp437";
		case CP_936:        *charset_len = sizeof("cp936")        - 1; return "cp437";
		case CP_950:        *charset_len = sizeof("cp950")        - 1; return "cp437";
		case CP_1125:       *charset_len = sizeof("cp1125")       - 1; return "cp437";
		case TIS:           *charset_len = sizeof("tis")          - 1; return "tis";
		case SHIFT_JIS:     *charset_len = sizeof("shift-jis")    - 1; return "shift-jis";
		case BIG5:          *charset_len = sizeof("big5")         - 1; return "big5";
		case EUC_KR:        *charset_len = sizeof("euc-kr")       - 1; return "euc-kr";
		case OTHER_CHARSET:
		default:
			return (const char *)zm_block_get_data(&(part -> raw_charset), msg, charset_len);
	}
/* Make compiler happy */
return NULL;
}

/*
 * Get next record or NULL, if no next record available
 */
ZM_KEY_VAL * zm_key_val_next(const ZM_KEY_VAL  * key_val)
{
	if (key_val == NULL) { return NULL; }

return key_val -> next;
}

/*
 * Get first field key-value pair
 */
ZM_KEY_VAL * zm_field_val_first(const ZM_FIELD_VAL * field_val)
{
	;;
return field_val -> pairs;
}

/*
 * Get first header os multipart or NULL, if no headers available
 */
ZM_FIELD_REC * zm_part_header_next(const ZM_FIELD_REC  * header)
{
	if (header == NULL) { return NULL; }

return header -> next;
}

/*
 * Get first header os multipart or NULL, if no headers available
 */
ZM_FIELD_REC * zm_part_first_header(const ZM_PART  * part)
{
	if (part == NULL) { return NULL; }

return part -> header.fields;
}

/*
 * Get next part
 */
ZM_PART * zm_next_part(const ZM_PART  * part)
{
	if (part == NULL) { return NULL; }
return part -> next;
}

/*
 * Get first subpart os multipart or NULL, if no record available
 */
ZM_PART * zm_multipart_first_subpart(const ZM_PART  * part)
{
	if (part == NULL || part -> type != MULTIPART) { return NULL; }

return part -> u.multi.parts;
}

/*
 * Get message subpart
 */
ZM_PART * zm_message_subpart(const ZM_PART  * part)
{
	if (part == NULL || part -> type != MESSAGE) { return NULL; }

return part -> u.message.part;
}

/**
  * @brief Free MIME part memrory and all subparts
  */
void zm_free_part(ZM_PART * part)
{
	ZM_ALLOCATOR  * alloc = part -> alloc;

	ZM_FIELD_REC  * field = zm_part_first_header(part);
	while (field != NULL)
	{
		ZM_FIELD_REC * next_field = zm_part_header_next(field);

		ZM_KEY_VAL * key_val = zm_field_val_first(&(field -> value));
		while (key_val != NULL)
		{
			ZM_KEY_VAL * next_key_val =  zm_key_val_next(key_val);

			zm_deallocate(alloc, key_val);

			key_val = next_key_val;
		}

		zm_deallocate(alloc, field);

		field = next_field;
	}

	if (part -> type == MULTIPART)
	{
		ZM_PART  * mpart = zm_multipart_first_subpart(part);
		while (mpart != NULL)
		{
			ZM_PART * next = zm_next_part(mpart);
			zm_free_part(mpart);
			mpart = next;
		}
	}
	else if (part -> type == MESSAGE)
	{
		zm_free_part(zm_message_subpart(part));
	}

	zm_deallocate(alloc, part);
}

/* End. */
