/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#include "zm_mem.h"
#include "zm_time_parser.h"

#include <string.h>
#include <strings.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <errno.h>

#define ABBREV_LINE_SIZE 512
#define ZM_DT_BUF_LEN    128

typedef struct _zm_abbrev_rec    ZM_ABBREV_REC;

struct _zm_abbrev_rec
{
	/** Abbreviation       */
	char            abbrev[16];
	/** Timezone offset    */
	int32_t         offset;
	/** Is Daylight or not */
	unsigned char   is_dst;
};

struct _zm_abbrev
{
	/** Abbreviation       */
	ZM_ABBREV_REC * abbrev;
	/** Timezone offset    */
	uint32_t        abbrev_count;
	/** Allocator          */
	ZM_ALLOCATOR  * alloc;
};

typedef struct _zm_abbrev_tmp    ZM_ABBREV_TMP;
struct _zm_abbrev_tmp
{
	/** Abbreviation              */
	ZM_ABBREV_REC    rec;
	/** Next abbreviation or NULL */
	ZM_ABBREV_TMP  * next;
};

static ZM_ABBREV_REC zm_default_abbrev_rec[] =
{
	{       "A",     3600, 1 },
	{    "ACDT",    37800, 1 },
	{   "ACSST",    37800, 1 },
	{    "ACST",    34200, 1 },
	{     "ACT",   -18000, 1 },
	{   "ACWST",    31500, 1 },
	{     "ADT",   -10800, 1 },
	{    "AEDT",    39600, 1 },
	{   "AESST",    39600, 1 },
	{    "AEST",    36000, 1 },
	{     "AFT",    16200, 1 },
	{    "AKDT",   -28800, 1 },
	{    "AKST",   -32400, 1 },
	{   "ALMST",    25200, 1 },
	{    "ALMT",    21600, 1 },
	{    "AMST",    14400, 1 },
	{     "AMT",   -14400, 1 },
	{   "ANAST",    43200, 1 },
	{    "ANAT",    43200, 1 },
	{    "ARST",   -10800, 1 },
	{     "ART",   -10800, 1 },
	{     "AST",   -14400, 1 },
	{   "AWSST",    32400, 1 },
	{    "AWST",    28800, 1 },
	{   "AZOST",        0, 1 },
	{    "AZOT",    -3600, 1 },
	{    "AZST",    14400, 1 },
	{     "AZT",    14400, 1 },
	{       "B",     7200, 1 },
	{    "BDST",     7200, 1 },
	{     "BDT",    21600, 1 },
	{     "BNT",    28800, 1 },
	{    "BORT",    28800, 1 },
	{     "BOT",   -14400, 1 },
	{     "BRA",   -10800, 1 },
	{    "BRST",    -7200, 1 },
	{     "BRT",   -10800, 1 },
	{     "BST",     3600, 1 },
	{     "BTT",    21600, 1 },
	{       "C",    10800, 1 },
	{    "CADT",    37800, 1 },
	{    "CAST",    34200, 1 },
	{     "CCT",    28800, 1 },
	{     "CDT",   -18000, 1 },
	{    "CEST",     7200, 1 },
	{     "CET",     3600, 1 },
	{  "CETDST",     7200, 1 },
	{   "CHADT",    49500, 1 },
	{   "CHAST",    45900, 1 },
	{    "CHUT",    36000, 1 },
	{     "CKT",   -36000, 1 },
	{    "CLST",   -10800, 1 },
	{     "CLT",   -14400, 1 },
	{     "COT",   -18000, 1 },
	{     "CST",   -21600, 1 },
	{     "CXT",    25200, 1 },
	{       "D",    14400, 1 },
	{    "DAVT",    25200, 1 },
	{    "DDUT",    36000, 1 },
	{       "E",    18000, 1 },
	{   "EASST",   -21600, 1 },
	{    "EAST",   -21600, 1 },
	{     "EAT",    10800, 1 },
	{     "EDT",   -14400, 1 },
	{    "EEST",    10800, 1 },
	{     "EET",     7200, 1 },
	{  "EETDST",    10800, 1 },
	{    "EGST",        0, 1 },
	{     "EGT",    -3600, 1 },
	{     "EST",   -18000, 1 },
	{      "ET",   -18000, 1 },
	{       "F",    21600, 1 },
	{     "FET",    10800, 1 },
	{    "FJST",    46800, 1 },
	{     "FJT",    43200, 1 },
	{    "FKST",   -10800, 1 },
	{     "FKT",   -10800, 1 },
	{    "FNST",    -3600, 1 },
	{     "FNT",    -7200, 1 },
	{       "G",    25200, 1 },
	{    "GALT",   -21600, 1 },
	{    "GAMT",   -32400, 1 },
	{    "GEST",    14400, 1 },
	{     "GET",    14400, 1 },
	{     "GFT",   -10800, 1 },
	{    "GILT",    43200, 1 },
	{     "GMT",        0, 1 },
	{     "GYT",   -14400, 1 },
	{       "H",    28800, 1 },
	{     "HKT",    28800, 1 },
	{     "HST",   -36000, 1 },
	{       "I",    32400, 1 },
	{     "ICT",    25200, 1 },
	{     "IDT",    10800, 1 },
	{     "IOT",    21600, 1 },
	{   "IRKST",    28800, 1 },
	{    "IRKT",    28800, 1 },
	{     "IRT",    12600, 1 },
	{     "IST",     7200, 1 },
	{    "JAYT",    32400, 1 },
	{     "JST",    32400, 1 },
	{       "K",    36000, 1 },
	{     "KDT",    36000, 1 },
	{    "KGST",    21600, 1 },
	{     "KGT",    21600, 1 },
	{    "KOST",    39600, 1 },
	{   "KRAST",    25200, 1 },
	{    "KRAT",    25200, 1 },
	{     "KST",    32400, 1 },
	{       "L",    39600, 1 },
	{    "LHDT",    37800, 1 },
	{    "LHST",    37800, 1 },
	{    "LIGT",    36000, 1 },
	{    "LINT",    50400, 1 },
	{     "LKT",    19800, 1 },
	{       "M",    43200, 1 },
	{   "MAGST",    39600, 1 },
	{    "MAGT",    39600, 1 },
	{    "MART",   -30600, 1 },
	{    "MAWT",    18000, 1 },
	{     "MDT",   -21600, 1 },
	{    "MEST",     7200, 1 },
	{    "MESZ",     7200, 1 },
	{     "MET",     3600, 1 },
	{  "METDST",     7200, 1 },
	{     "MEZ",     3600, 1 },
	{     "MHT",    43200, 1 },
	{     "MMT",    23400, 1 },
	{     "MPT",    36000, 1 },
	{     "MSD",    14400, 1 },
	{     "MSK",    10800, 1 },
	{     "MST",   -25200, 1 },
	{    "MUST",    18000, 1 },
	{     "MUT",    14400, 1 },
	{     "MVT",    18000, 1 },
	{     "MYT",    28800, 1 },
	{       "N",    -3600, 1 },
	{     "NDT",    -5400, 1 },
	{     "NFT",    -9000, 1 },
	{   "NOVST",    25200, 1 },
	{    "NOVT",    25200, 1 },
	{     "NPT",    20700, 1 },
	{     "NST",    -9000, 1 },
	{     "NUT",   -39600, 1 },
	{    "NZDT",    46800, 1 },
	{    "NZST",    43200, 1 },
	{     "NZT",    43200, 1 },
	{       "O",    -7200, 1 },
	{   "OMSST",    21600, 1 },
	{    "OMST",    21600, 1 },
	{       "P",   -10800, 1 },
	{     "PDT",   -25200, 1 },
	{     "PET",   -18000, 1 },
	{   "PETST",    43200, 1 },
	{    "PETT",    43200, 1 },
	{     "PGT",    36000, 1 },
	{    "PHOT",    46800, 1 },
	{     "PHT",    28800, 1 },
	{    "PKST",    21600, 1 },
	{     "PKT",    18000, 1 },
	{    "PMDT",    -7200, 1 },
	{    "PMST",   -10800, 1 },
	{    "PONT",    39600, 1 },
	{     "PST",   -28800, 1 },
	{     "PWT",    32400, 1 },
	{    "PYST",   -10800, 1 },
	{     "PYT",   -14400, 1 },
	{       "Q",   -14400, 1 },
	{       "R",   -18000, 1 },
	{     "RET",    14400, 1 },
	{       "S",   -21600, 1 },
	{    "SADT",    37800, 1 },
	{    "SAST",     7200, 1 },
	{     "SCT",    14400, 1 },
	{     "SGT",    28800, 1 },
	{       "T",   -25200, 1 },
	{    "TAHT",   -36000, 1 },
	{     "TFT",    18000, 1 },
	{     "TJT",    18000, 1 },
	{     "TKT",    46800, 1 },
	{     "TMT",    18000, 1 },
	{     "TOT",    46800, 1 },
	{    "TRUT",    36000, 1 },
	{     "TVT",    43200, 1 },
	{       "U",   -28800, 1 },
	{     "UCT",        0, 1 },
	{   "ULAST",    32400, 1 },
	{    "ULAT",    28800, 1 },
	{      "UT",        0, 1 },
	{     "UTC",        0, 1 },
	{    "UYST",    -7200, 1 },
	{     "UYT",   -10800, 1 },
	{    "UZST",    21600, 1 },
	{     "UZT",    18000, 1 },
	{       "V",   -32400, 1 },
	{     "VET",   -14400, 1 },
	{   "VLAST",    36000, 1 },
	{    "VLAT",    36000, 1 },
	{    "VOLT",    10800, 1 },
	{     "VUT",    39600, 1 },
	{       "W",   -36000, 1 },
	{    "WADT",    28800, 1 },
	{    "WAKT",    43200, 1 },
	{    "WAST",    25200, 1 },
	{     "WAT",     3600, 1 },
	{     "WDT",    32400, 1 },
	{     "WET",        0, 1 },
	{  "WETDST",     3600, 1 },
	{     "WFT",    43200, 1 },
	{    "WGST",    -7200, 1 },
	{     "WGT",   -10800, 1 },
	{       "X",   -39600, 1 },
	{     "XJT",    21600, 1 },
	{       "Y",   -43200, 1 },
	{   "YAKST",    32400, 1 },
	{    "YAKT",    32400, 1 },
	{    "YAPT",    36000, 1 },
	{   "YEKST",    21600, 1 },
	{    "YEKT",    18000, 1 },
	{       "Z",        0, 1 },
	{    "ZULU",        0, 1 },
};
static uint32_t zm_default_abbrev_rec_count = sizeof(zm_default_abbrev_rec);

static void zm_close_abbrev(FILE           * f,
                            ZM_ALLOCATOR   * alloc,
                            ZM_ABBREV_TMP  * abbrev,
                            int32_t          err_no)
{
	fclose(f);
	while (abbrev != NULL)
	{
		ZM_ABBREV_TMP * tmp = abbrev -> next;
		zm_deallocate(alloc, abbrev);
		abbrev = tmp;
	}
	errno = err_no;
}

/*
 * Sorting functor
 */
static int zm_abbrev_cmp(const void * x, const void * y)
{
	ZM_ABBREV_REC  * rec_x = (ZM_ABBREV_REC *)x;
	ZM_ABBREV_REC  * rec_y = (ZM_ABBREV_REC *)y;

return strcasecmp(rec_x -> abbrev, rec_y -> abbrev);
}

/*
 * Load custom timezone abbreviatures
 */
int32_t zm_load_abbrev(const char     * file,
                       ZM_ALLOCATOR   * alloc,
                       ZM_ABBREV     ** abbrev)
{
	ZM_ABBREV_REC  * abbrev_rec     = NULL;
	ZM_ABBREV_TMP  * abbrev_tmp_st  = NULL;
	ZM_ABBREV_TMP  * abbrev_tmp_cur = NULL;

	int32_t count = 0;

	FILE * f = fopen(file, "r");
	if (f == NULL) { return -1; }

	for(;;)
	{
		ZM_ABBREV_TMP  * tmp = NULL;
		char     buf[ABBREV_LINE_SIZE + 1];
		char     tz_name[ABBREV_LINE_SIZE];
		int32_t  h = 0;
		int32_t  m = 0;
		int32_t  s = 0;
		char     is_dst = 0;
		int32_t  addr_len = 0;

		if (fgets(buf, ABBREV_LINE_SIZE, f) == NULL) { break; }

		if (buf[0] == '#' || buf[0] == '\r' || buf[0] == '\n' || buf[0] == '\0') { continue; }

		/* ACDT    10:30:00 t */
		if (sscanf(buf, "%s %d:%d:%d %c", tz_name, &h, &m, &s, &is_dst) != 5)
		{
			zm_close_abbrev(f, alloc, abbrev_tmp_st, EINVAL);
			return -1;
		}

		addr_len = strlen(tz_name);
		if (addr_len > 15)
		{
			zm_close_abbrev(f, alloc, abbrev_tmp_st, EINVAL);
			return -1;
		}

		tmp = zm_allocate(alloc, sizeof(ZM_ABBREV_TMP));
		if (tmp == NULL)
		{
			zm_close_abbrev(f, alloc, abbrev_tmp_st, ENOMEM);
			return -1;
		}

		if (abbrev_tmp_st == NULL)
		{
			abbrev_tmp_st = abbrev_tmp_cur = tmp;
		}
		else
		{
			abbrev_tmp_cur -> next = tmp;
			abbrev_tmp_cur = tmp;
		}

		++count;

		strncpy(abbrev_tmp_cur -> rec.abbrev, tz_name, addr_len + 1);
		abbrev_tmp_cur -> rec.offset = h * 3600 + m * 60 + s;
		abbrev_tmp_cur -> rec.is_dst = is_dst == 't' ? 0 : 1;
	}

	*abbrev = zm_allocate(alloc, sizeof(ZM_ABBREV));
	if (*abbrev == NULL)
	{
		zm_close_abbrev(f, alloc, abbrev_tmp_st, ENOMEM);
		return -1;
	}

	(*abbrev) -> alloc = alloc;
	(*abbrev) -> abbrev_count = count;
	(*abbrev) -> abbrev       = abbrev_rec = zm_allocate(alloc, count * sizeof(ZM_ABBREV_REC));
	if ((*abbrev) -> abbrev == NULL)
	{
		zm_close_abbrev(f, alloc, abbrev_tmp_st, ENOMEM);
		zm_deallocate(alloc, (*abbrev));
		return -1;
	}

	while (abbrev_tmp_st != NULL)
	{
		ZM_ABBREV_TMP * tmp = abbrev_tmp_st -> next;

		memcpy(abbrev_rec, &(abbrev_tmp_st -> rec), sizeof(ZM_ABBREV_REC));

		zm_deallocate(alloc, abbrev_tmp_st);
		abbrev_tmp_st = tmp;
		++abbrev_rec;
	}

	qsort((*abbrev) -> abbrev, count, sizeof(ZM_ABBREV_REC), zm_abbrev_cmp);

	fclose(f);
return 0;
}

/*
 * Print custom timezone abbreviatures
 */
void zm_print_abbrev(ZM_ABBREV  * abbrev)
{
	uint32_t pos = 0;

	ZM_ABBREV_REC * rec = abbrev -> abbrev;
	for(pos = 0; pos < abbrev -> abbrev_count; ++pos)
	{
		fprintf(stdout, "{ %7s, %8d, %d }\n", rec -> abbrev, rec -> offset, rec -> is_dst);
		++rec;
	}
}

/*
 * Print custom timezone abbreviatures
 */
void zm_free_abbrev(ZM_ABBREV  * abbrev)
{
	ZM_ALLOCATOR  * alloc = abbrev -> alloc;
	zm_deallocate(alloc, abbrev -> abbrev);
	zm_deallocate(alloc, abbrev);
}

/*
 * Get timezone offset
 */
static ZM_ABBREV_REC * zm_get_zone_offset(const char     * tz,
                                          ZM_ABBREV_REC  * abbrev_rec,
                                          uint32_t         abbrev_rec_count)
{
	int32_t l_pos = 0;
	int32_t r_pos = abbrev_rec_count;
	int32_t pos = 0;
	int32_t rc = 0;
	for(;;)
	{
		pos = (l_pos + r_pos) / 2;
		rc = strcasecmp(abbrev_rec[pos].abbrev, tz);

		if (pos == l_pos || pos == r_pos) { return NULL; }

		if      (rc == 0) { break; }
		else if (rc <  0) { l_pos = pos; }
		else              { r_pos = pos; }
	}

return &abbrev_rec[pos];
}

/*
 * Parse datetime string
 */
int32_t zm_parse_datetime(const char      * dt,
                          const uint32_t    dt_len,
                          int64_t         * ts,
                          int64_t         * ts_offset,
                          ZM_ABBREV       * abbrev)
{
	char     sign   = 0;
	int      offt_h = 0;
	int      offt_m = 0;
	int      offt_s = 0;
	int32_t  elts   = 0;
	char dt_buf[ZM_DT_BUF_LEN + 1];

	char month[10];
	char wday[10];

	struct tm tm;

	ZM_ABBREV_REC * rec = NULL;
	ZM_ABBREV_REC * abbrev_rec       = zm_default_abbrev_rec;
	uint32_t        abbrev_rec_count = zm_default_abbrev_rec_count / sizeof(ZM_ABBREV_REC);

	if (abbrev != NULL)
	{
		abbrev_rec = abbrev -> abbrev;
		abbrev_rec_count = abbrev -> abbrev_count;
	}

	memset(&tm, 0, sizeof(struct tm));

	if (dt_len >= ZM_DT_BUF_LEN) { return -1; }

	memcpy(dt_buf, dt, dt_len);
	dt_buf[dt_len] = 0;

	wday[0] = month[0] = 0; /* make valgrind happy */

	/* "Tue, 12 Oct 2010 14:33:16 +0000" */
	elts = sscanf(dt_buf, "%3s, %d %3s %d %02d:%02d:%02d %c%02d%02d", (char*)(&wday), &tm.tm_mday, (char*)(&month), &tm.tm_year, &tm.tm_hour, &tm.tm_min, &tm.tm_sec, &sign, &offt_h, &offt_m);
	if (elts != 10)
	{
		char tz[7];

		sign = '+';
		/* "Tue, 12 Oct 2010 14:33:16 CET" */
		elts = sscanf(dt_buf, "%3s, %d %3s %d %02d:%02d:%02d %6s", (char*)(&wday), &tm.tm_mday, (char*)(&month), &tm.tm_year, &tm.tm_hour, &tm.tm_min, &tm.tm_sec, (char*)(&tz));
		if (elts == 8)
		{
			rec = zm_get_zone_offset(tz, abbrev_rec, abbrev_rec_count);
			if (rec != NULL)
			{
				offt_s      = rec -> offset;
				tm.tm_isdst = rec -> is_dst;
			}
		}
		else
		{
			/* "Tue Jun 19 17:33:03 UTC 2018" */
			elts = sscanf(dt_buf, "%3s %3s %d %02d:%02d:%02d %6s %d", (char*)(&wday), (char*)(&month), &tm.tm_mday, &tm.tm_hour, &tm.tm_min, &tm.tm_sec, (char*)(&tz), &tm.tm_year);
			if (elts == 8)
			{
				rec = zm_get_zone_offset(tz, abbrev_rec, abbrev_rec_count);
				if (rec != NULL)
				{
					offt_s      = rec -> offset;
					tm.tm_isdst = rec -> is_dst;
				}
			}
			else
			{
				/* "31 Dec 2008 18:25:44 +0300" */
				elts = sscanf(dt_buf, "%d %3s %d %02d:%02d:%02d %c%02d%02d", &tm.tm_mday, (char*)(&month), &tm.tm_year, &tm.tm_hour, &tm.tm_min, &tm.tm_sec, &sign, &offt_h, &offt_m);
				if (elts != 9)
				{
					/* "31 Dec 2008 18:25:44 CET" */
					elts = sscanf(dt_buf, "%d %3s %d %02d:%02d:%02d %6s", &tm.tm_mday, (char*)(&month), &tm.tm_year, &tm.tm_hour, &tm.tm_min, &tm.tm_sec, (char*)(&tz));
					if (elts == 7)
					{
						rec = zm_get_zone_offset(tz, abbrev_rec, abbrev_rec_count);
						if (rec != NULL)
						{
							offt_s = rec -> offset;
							tm.tm_isdst = rec -> is_dst;
						}
					}
					else { memset(&tm, 0, sizeof(struct tm)); }
				}
			}
		}
	}

	if (tm.tm_year > 1900) { tm.tm_year -= 1900; }

	switch(month[0] | 0x20)
	{
		/* Jan, Jun, Jul */
		case 'j':
			{
				if      ((month[1] | 0x20) == 'a') { tm.tm_mon = 0; }
				else if ((month[2] | 0x20) == 'n') { tm.tm_mon = 5; }
				else                               { tm.tm_mon = 6; }
			}
			break;
		/* Mar May */
		case 'm':
			{
				if ((month[2] | 0x20) == 'r') { tm.tm_mon = 2; }
				else                          { tm.tm_mon = 4; }
			}
			break;
		/* Apr, Aug */
		case 'a':
			{
				if ((month[1] | 0x20) == 'p') { tm.tm_mon = 3; }
				else                          { tm.tm_mon = 7; }
			}
			break;
		/* Feb */
		case 'f': tm.tm_mon = 1; break;
		/* Sep */
		case 's': tm.tm_mon = 8; break;
		/* Oct */
		case 'o': tm.tm_mon = 9; break;
		/* Nov */
		case 'n': tm.tm_mon = 10; break;
		/* Dec */
		case 'd': tm.tm_mon = 11; break;
	}

	*ts = mktime(&tm);
	*ts_offset   = ((sign == '-') ? -1 : 1) * (offt_h * 3600 + offt_m * 60) + offt_s;

	*ts -= *ts_offset;

return 0;
}
/* End. */
