/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#include <stdio.h>
#include <string.h>
#include <sysexits.h>

#include "zm_address_parser.h"
#include "zm_data_buffer_impl.h"
#include "zm_email_address.h"
#include "zm_iconv.h"
#include "zm_mem.h"

static void print_address(ZM_EMAIL_ADDRESS  * email_address)
{
	while (email_address != NULL)
	{
		fprintf(stdout, "g: `%.*s`, n: `%.*s`, r: `%.*s`, u: `%.*s`, d: `%.*s`,\n",
		                 (int32_t)zm_data_buffer_get_size(&(email_address -> group)),    zm_data_buffer_get(&(email_address -> group)),
		                 (int32_t)zm_data_buffer_get_size(&(email_address -> name)),     zm_data_buffer_get(&(email_address -> name)),
		                 (int32_t)zm_data_buffer_get_size(&(email_address -> route)),    zm_data_buffer_get(&(email_address -> route)),
		                 (int32_t)zm_data_buffer_get_size(&(email_address -> username)), zm_data_buffer_get(&(email_address -> username)),
		                 (int32_t)zm_data_buffer_get_size(&(email_address -> domain)),   zm_data_buffer_get(&(email_address -> domain)));

		email_address = zm_email_address_next(email_address);
	}
}

int main(int argc, char ** argv)
{
	ZM_ICONV               iconv;
	ZM_DATA_BUFFER         buffer;
	ZM_ADDRESS_PARSER      address_parser;
	ZM_EMAIL_ADDRESS     * email_address = NULL;
	const unsigned char  * address = NULL;
	uint64_t               address_len = 0;

	if (argc != 2) { fprintf(stderr, "usage: %s '\"John Doe\" <jd@example.com>'\n", argv[0]); return EX_USAGE; }

	address =  (const unsigned char *)argv[1];
	address_len = strlen(argv[1]);

	zm_data_buffer_create_impl(&buffer, &zm_default_allocator);
	zm_iconv_create(&iconv, &zm_default_allocator);

	zm_address_parser_create(&address_parser, &iconv, &buffer);

	zm_address_parser_parse_address_list(&address_parser, &address, &address_len, &email_address);

	print_address(email_address);

	zm_email_address_destroy_chain(email_address);

	zm_address_parser_destroy(&address_parser);
	zm_iconv_destroy(&iconv);
	zm_data_buffer_destroy(&buffer);

return 0;
}
/* End. */
