/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>

#include <fcntl.h>
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include <strings.h>
#include <sysexits.h>
#include <unistd.h>

#include "zm_charset_detector.h"

int main(int argc, char ** argv)
{
	int32_t           fh      = -1;
	void            * msg     = NULL;
	uint32_t          msg_len = 0;
	enum ZM_CHARSET  chrst = OTHER_CHARSET;
	struct stat      st;
	int32_t          code = EX_OK;

	if (argc != 2 && argc != 3) { fprintf(stderr, "usage: %s text [charset]\n", argv[0]); return EX_USAGE; }

	fh = open(argv[1], O_NONBLOCK, O_RDONLY);
	if (fh == -1)
	{
		fprintf(stderr, "Can't open file `%s`\n", argv[1]);
		return EX_SOFTWARE;
	}

	if (fstat(fh, &st) == -1)
	{
		fprintf(stderr, "Can't stat file `%s`\n", argv[1]);
		close(fh);
		return EX_SOFTWARE;
	}

	msg_len = st.st_size;
	msg     = mmap(NULL, msg_len, PROT_READ, MAP_PRIVATE, fh, 0);
	if (msg == MAP_FAILED)
	{
		fprintf(stderr, "Can't map file `%s` into memory\n", argv[1]);
		close(fh);
		return EX_SOFTWARE;
	}

	fprintf(stdout, "Charset: ");
	chrst = zm_detect_charset((const unsigned char *)msg, msg_len);
	switch(chrst)
	{
		case UTF_8:        fprintf(stdout, "UTF-8\n"); break;
		case WINDOWS_1251: fprintf(stdout, "CP1251\n"); break;
		case KOI_8R:       fprintf(stdout, "KOI\n"); break;
		case US_ASCII:     fprintf(stdout, "US_ASCII\n"); break;
		default:
			fprintf(stdout, "OTHER/%u\n", chrst);
	}

	if (argc == 3)
	{
		code = EX_SOFTWARE;
		if      (chrst == UTF_8        && strcmp("UTF-8",    argv[2]) == 0) { code = EX_OK; }
		else if (chrst == WINDOWS_1251 && strcmp("CP1251",   argv[2]) == 0) { code = EX_OK; }
		else if (chrst == KOI_8R       && strcmp("KOI8-R",   argv[2]) == 0) { code = EX_OK; }
		else if (chrst == US_ASCII     && strcmp("US-ASCII", argv[2]) == 0) { code = EX_OK; }
	}

	munmap(msg, msg_len);
	close(fh);
return code;
}

