/*-
 *  Copyright (c) Iris Dev. team. All rights reserved.
 *  See http://www.communico.pro/products/zeromime/license/ for details.
 *
 */
#include <stdio.h>
#include <string.h>
#include <sysexits.h>

#include "zm_block.h"
#include "zm_data_buffer_impl.h"
#include "zm_mem.h"
#include "zm_mime_converter.h"

static void base64_test(const char * src)
{
	uint64_t               curr_len = strlen(src);
	const unsigned char  * curr     = (const unsigned char *)src;

	for(;;)
	{
		unsigned char dst[12];
		uint64_t      dst_len = 0;

		const unsigned char * res = zm_decode_base64_block(curr, curr_len, dst, &dst_len, 12, zm_base64_rev);

		fprintf(stdout, "%.*s", (int32_t)dst_len, (const char *)dst);

		curr_len -= (res - curr);
		curr = res;
		if (curr_len == 0) { break; }
	}
	fprintf(stdout, "\n\n");
}

static void base64_data_buffer_test(const char * src, ZM_DATA_BUFFER  * dst)
{
	const char  * dst_str  = NULL;
	int32_t       dst_len = 0;
	ZM_BLOCK      src_block;

	zm_block_create(&src_block, 0, strlen(src));

	zm_decode_base64(&src_block, src, zm_base64_rev, dst);

	dst_str = zm_data_buffer_get(dst);
	dst_len = zm_data_buffer_get_size(dst);

	fprintf(stdout, "%.*s\n\n", dst_len, dst_str);
}

static void quoted_printable_test(const char * src)
{
	uint64_t               curr_len = strlen(src);
	const unsigned char  * curr     = (const unsigned char *)src;

	for(;;)
	{
		unsigned char dst[12];
		uint64_t      dst_len = 0;

		const unsigned char * res = zm_decode_quoted_printable_block(curr, curr_len, dst, &dst_len, 12);

		fprintf(stdout, "%.*s", (int32_t)dst_len, (const char *)dst);

		curr_len -= (res - curr);
		curr = res;
		if (curr_len == 0) { break; }
	}
	fprintf(stdout, "\n\n");
}

static void quoted_printable_data_buffer_test(const char * src, ZM_DATA_BUFFER  * dst)
{
	const char  * dst_str  = NULL;
	int32_t       dst_len = 0;
	ZM_BLOCK      src_block;

	zm_block_create(&src_block, 0, strlen(src));

	zm_decode_quoted_printable(&src_block, src, dst);

	dst_str = zm_data_buffer_get(dst);
	dst_len = zm_data_buffer_get_size(dst);

	fprintf(stdout, "%.*s\n\n", dst_len, dst_str);
}

static void qencoded_test(const char * src)
{
	uint64_t               curr_len = strlen(src);
	const unsigned char  * curr     = (const unsigned char *)src;

	for(;;)
	{
		unsigned char dst[12];
		uint64_t      dst_len = 0;

		const unsigned char * res = zm_decode_qencoded_block(curr, curr_len, dst, &dst_len, 12);

		fprintf(stdout, "%.*s", (int32_t)dst_len, (const char *)dst);

		curr_len -= (res - curr);
		curr = res;
		if (curr_len == 0) { break; }

		if (*res == '?') { break; }
	}
	fprintf(stdout, "\n\n");
}

static void qencoded_data_buffer_test(const char * src, ZM_DATA_BUFFER  * dst)
{
	const char  * dst_str  = NULL;
	int32_t       dst_len = 0;
	ZM_BLOCK      src_block;

	zm_block_create(&src_block, 0, strlen(src));

	zm_decode_qencoded(&src_block, src, dst);

	dst_str = zm_data_buffer_get(dst);
	dst_len = zm_data_buffer_get_size(dst);

	fprintf(stdout, "%.*s\n\n", dst_len, dst_str);
}

static const char * base64_src = "QmFzZTY0IGlzIGEg Z3JvdXAgb2\001Ygc2ltaWxhciBiaW5hcnktdG8tdGV4dCBlbmNv "
                                 "ZGluZyBzY2h lbWVzIHRoYXQgcmVwcmVzZW50IGJpbmFyeSBkYXRhIGluIGFuIEFT\t"
                                 "Q0lJIHN 0cmluZyBmb3JtYXQgYnkgdHJhbnNsYXRpbmcgaXQgaW50byBhIHJhZGl4\n"
                                 "LTY0 IHJlcHJlc2V ud GF0aW9uLiA=";

static const char * quoted_printable_src = "=D0=AD=D1=82=D0=BE=D1=82 =D0=BC=D0=B5=D1=85=D0=B0=D0=BD=D0=B8=D0=B7=D0=BC =\n"
                                           "=D0=BF=D1=80=D0=B5=D0=B4=D0=BD=D0=B0=D0=B7=D0=BD=D0=B0=D1=87=D0=B5=D0=BD =\n"
                                           "=D0=B4=D0=BB=D1=8F =D0=BF=D1=80=D0=B5=D0=B4=D1=81=D1=82=D0=B0=D0=B2=D0=BB=\n"
                                           "=D0=B5=D0=BD=D0=B8=D1=8F =D0=B4=D0=B0=D0=BD=D0=BD=D1=8B=D1=85, =D0=B2 =D0=\n"
                                           "=BE=D1=81=D0=BD=D0=BE=D0=B2=D0=BD=D0=BE=D0=BC =D1=81=D0=BE=D1=81=D1=82=D0=\n"
                                           "=BE=D1=8F=D1=89=D0=B8=D1=85 =D0=B8=D0=B7 =D0=B1=D0=B0=D0=B9=D1=82=D0=BE=D0=\n"
                                           "=B2.=20";

static const char * qencoded_src = "=D0=95=D1=81=D0=BB=D0=B8_=D1=83=D0=B6_=D1=86=D0=B2=D0=B5=D1=82endofspeech";

static const char * result = "Data buffer test: Base64 is a group of similar binary-to-text encoding schemes that "
                             "represent binary data in an ASCII string format by translating it into a radix-64 representation. "
                             "Этот механизм предназначен для представления данных, в основном состоящих из байтов. Если уж цветendofspeech";

int main(int argc, char ** argv)
{
	ZM_DATA_BUFFER    dst;
	const char      * dst_str  = NULL;
	int32_t           dst_len = 0;
	int32_t           code    = EX_SOFTWARE;

	if (argc != 1) { fprintf(stderr, "usage: %s\n", argv[0]); return EX_USAGE; }

	zm_data_buffer_create_impl(&dst, &zm_default_allocator);
	zm_data_buffer_append_str(&dst, "Data buffer test: ");

	base64_test(base64_src);

	base64_data_buffer_test(base64_src, &dst);

	quoted_printable_test(quoted_printable_src);

	quoted_printable_data_buffer_test(quoted_printable_src, &dst);

	qencoded_test(qencoded_src);

	qencoded_data_buffer_test(qencoded_src, &dst);

	dst_str = zm_data_buffer_get(&dst);
	dst_len = zm_data_buffer_get_size(&dst);

	if (strncmp(dst_str, result, dst_len) == 0) { code = EX_OK; }

	zm_data_buffer_destroy(&dst);
return code;
}
/* End. */
